@@ -147,7 +147,7 @@
 
 ;;; Return the length of VECTOR.
 ;;; Ordinary code should prefer to use (LENGTH (THE VECTOR FOO)) instead.
-(defknown vector-length (vector) index (flushable dx-safe))
+(defknown vector-length ((read-only vector)) index (flushable dx-safe))
 
 (defknown vector-sap ((simple-unboxed-array (*))) system-area-pointer
   (flushable))
@@ -203,6 +203,7 @@
 (defknown sb-kernel::check-array-shape (simple-array list)
   (simple-array)
   (flushable no-verify-arg-count)
+  :derive-type #'result-type-first-arg
   :result-arg 0)
 
 (defknown (%make-instance %make-instance/mixed) (index) instance
