@@ -98,6 +98,18 @@
   ;; flushed in some circumstances.
   (flushable nil :type list))
 
+#+sb-eval
+(defstruct (sb-eval::eval-lexenv
+            (:include lexenv)
+            (:copier nil)
+            (:constructor sb-eval::make-eval-lexenv
+                (funs vars blocks tags
+                 type-restrictions
+                 flushable
+                 lambda cleanup handled-conditions
+                 disabled-package-locks %policy user-data
+                 parent))))
+
 (defun lexenv-policy (lexenv)
   (or (lexenv-%policy lexenv) *policy*))
 
@@ -141,7 +153,7 @@
 ;;; continuations throughout the compiler.
 
 ;;; "Lead-in" Control TRANsfer [to some node]
-(defstruct (ctran (:constructor make-ctran) (:copier nil))
+(defstruct (ctran (:constructor make-ctran (&optional kind)) (:copier nil))
   ;; an indication of the way that this continuation is currently used
   ;;
   ;; :UNUSED
@@ -209,7 +221,8 @@
 ;;; (type derivation, runtime errors).
 ;;; Right now it's basically used for tracking constants and checking
 ;;; them for things like proper sequence, or valid type specifier.
-(defstruct lvar-annotation
+(defstruct (lvar-annotation
+            (:constructor nil))
   (source-path nil :type list)
   lexenv
   fired)
@@ -397,8 +410,8 @@
 
 ;;; The LOOP structure holds information about a loop.
 (defstruct (cloop (:conc-name loop-)
-                  (:predicate loop-p)
-                  (:constructor make-loop)
+                  (:predicate nil)
+                  (:constructor make-loop (kind head &optional tail))
                   (:copier nil))
   ;; The kind of loop that this is.  These values are legal:
   ;;
@@ -414,7 +427,7 @@
   (kind (missing-arg) :type (member :outer :natural :strange))
   ;; The first and last blocks in the loop.  There may be more than one tail,
   ;; since there may be multiple back branches to the same head.
-  (head nil :type (or cblock null))
+  (head (missing-arg) :type cblock :read-only t)
   (tail nil :type list)
   ;; A list of all the blocks in this loop or its inferiors that have a
   ;; successor outside of the loop.
@@ -447,7 +460,7 @@
 ;;; of the function.)
 (defstruct (cblock (:include sset-element)
                    (:constructor make-block (start))
-                   (:constructor make-block-key)
+                   (:constructor make-block-key (&key component pred succ start last))
                    (:copier nil)
                    (:conc-name block-)
                    (:predicate block-p))
@@ -472,7 +485,7 @@
   (next nil :type (or null cblock))
   (prev nil :type (or null cblock))
   ;; This block's attributes: see above.
-  (flags (block-attributes reoptimize flush-p type-check)
+  (flags #.(block-attributes reoptimize flush-p type-check)
          :type attributes)
   ;; in constraint propagation: list of LAMBDA-VARs killed in this block
   ;; in copy propagation: list of killed TNs
@@ -510,25 +523,21 @@
 (defmethod print-object ((cblock cblock) stream)
   (if (boundp '*compilation*)
       (print-unreadable-object (cblock stream :type t :identity t)
-        (format stream "~W :START c~W"
-            (block-number cblock)
-            (cont-num (block-start cblock))))
+        (let ((component (block-component cblock)))
+          (cond ((and component
+                      (eq cblock
+                          (component-head component)))
+                 (format stream "~W :HEAD" (block-number cblock)))
+                ((and component
+                      (eq cblock
+                          (component-tail component)))
+                 (format stream "~W :TAIL" (block-number cblock)))
+                (t
+                 (format stream "~W :START c~W"
+                         (block-number cblock)
+                         (cont-num (block-start cblock)))))))
       (print-unreadable-object (cblock stream :type t :identity t))))
 
-;;; The BLOCK-ANNOTATION class is inherited (via :INCLUDE) by
-;;; different BLOCK-INFO annotation structures so that code
-;;; (specifically control analysis) can be shared.
-(defstruct (block-annotation (:constructor nil)
-                             (:copier nil))
-  ;; The IR1 block that this block is in the INFO for.
-  (block (missing-arg) :type cblock)
-  ;; the next and previous block in emission order (not DFO). This
-  ;; determines which block we drop though to, and is also used to
-  ;; chain together overflow blocks that result from splitting of IR2
-  ;; blocks in lifetime analysis.
-  (next nil :type (or block-annotation null))
-  (prev nil :type (or block-annotation null)))
-
 ;;; A COMPONENT structure provides a handle on a connected piece of
 ;;; the flow graph. Most of the passes in the compiler operate on
 ;;; COMPONENTs rather than on the entire flow graph.
@@ -542,7 +551,10 @@
 ;;;   component.
 (defstruct (component (:copier nil)
                       (:constructor make-component
-                       (head tail &aux (last-block tail))))
+                       (head tail &aux (last-block tail)
+                                       (outer-loop
+                                        (make-loop :outer head (list tail)))))
+                      #-sb-xc-host :no-constructor-defun)
   ;; space where this component will be allocated in
   ;; :auto won't make any codegen optimizations pertinent to immobile space,
   ;; but will place the code there given sufficient available space.
@@ -583,8 +595,8 @@
   ;; tail. Until environment analysis links NLX entry stubs to the
   ;; component head, every successor of the head is a function start
   ;; (i.e. begins with a BIND node.)
-  (head (missing-arg) :type cblock)
-  (tail (missing-arg) :type cblock)
+  (head (missing-arg) :type cblock :read-only t)
+  (tail (missing-arg) :type cblock :read-only t)
   ;; New blocks are inserted before this.
   (last-block (missing-arg) :type cblock)
   ;; This becomes a list of the CLAMBDA structures for all functions
@@ -613,6 +625,9 @@
   ;; could benefit from further IR1 optimization. T means that
   ;; reoptimization is necessary.
   (reoptimize t :type (member nil :maybe t))
+  ;; For delay-ir1-transform to know when to delay
+  (reoptimize-counter 0 :type fixnum)
+  (phase-counter 0 :type fixnum)
   ;; If this is true, then the control flow in this component was
   ;; messed up by IR1 optimizations, so the DFO should be recomputed.
   (reanalyze nil :type boolean)
@@ -634,7 +649,7 @@
   (reanalyze-functionals nil :type list)
   (delete-blocks nil :type list)
   ;; The default LOOP in the component.
-  (outer-loop (make-loop :kind :outer :head head :tail (list tail)) :type cloop)
+  (outer-loop (missing-arg) :type cloop)
   (max-block-number 0 :type fixnum)
   (dominators-computed nil))
 
@@ -671,7 +686,9 @@
 ;;; boundaries by requiring that the exit ctrans initially head their
 ;;; blocks, and then by not merging blocks when there is a cleanup
 ;;; change.
-(defstruct (cleanup (:copier nil))
+(defstruct (cleanup (:copier nil)
+                    (:constructor make-cleanup
+                        (kind &optional mess-up)))
   ;; the kind of thing that has to be cleaned up
   (kind (missing-arg)
         :type (member :special-bind :catch :unwind-protect
@@ -690,7 +707,9 @@
   (nlx-info :test nlx-info))
 
 ;;; The ENVIRONMENT structure represents the result of environment analysis.
-(defstruct (environment (:copier nil))
+(defstruct (environment (:copier nil)
+                        (:constructor make-environment (lambda))
+                        #-sb-xc-host :no-constructor-defun)
   ;; the function that allocates this environment
   (lambda (missing-arg) :type clambda :read-only t)
   ;; a list of all the LAMBDA-VARs and NLX-INFOs needed from enclosing
@@ -719,7 +738,8 @@
 ;;; The tail set is somewhat approximate, because it is too early to
 ;;; be sure which calls will be tail-recursive. Any call that *might*
 ;;; end up tail-recursive causes TAIL-SET merging.
-(defstruct (tail-set)
+(defstruct (tail-set
+            (:constructor make-tail-set (funs)))
   ;; a list of all the LAMBDAs in this tail set
   (funs nil :type list)
   ;; our current best guess of the type returned by these functions.
@@ -739,7 +759,8 @@
 ;;; ENVIRONMENT-NLX-INFO.
 (defstruct (nlx-info
             (:copier nil)
-            (:constructor make-nlx-info (cleanup block)))
+            (:constructor make-nlx-info (cleanup block))
+            #-sb-xc-host :no-constructor-defun)
   ;; the cleanup associated with this exit. In a catch or
   ;; unwind-protect, this is the :CATCH or :UNWIND-PROTECT cleanup,
   ;; and not the cleanup for the escape block. The CLEANUP-KIND of
@@ -808,7 +829,7 @@
                 ;; I guess we state the type this way to avoid calling
                 ;; LEGAL-FUN-NAME-P unless absolutely necessary,
                 ;; but this seems a bit of a premature optimization.
-                :type (or symbol (and cons (satisfies legal-fun-name-p)))
+                :type (or symbol (and cons #-host-quirks-cmu (satisfies legal-fun-name-p)))
                 :read-only t)
   ;; the type which values of this leaf must have
   (type *universal-type* :type ctype)
@@ -817,11 +838,12 @@
   (defined-type *universal-type* :type ctype)
   ;; where the TYPE information came from (in order, from strongest to weakest):
   ;;  :DECLARED, from a declaration.
+  ;;  :DECLARED-VERIFY
   ;;  :DEFINED-HERE, from examination of the definition in the same file.
   ;;  :DEFINED, from examination of the definition elsewhere.
   ;;  :DEFINED-METHOD, implicit, piecemeal declarations from CLOS.
   ;;  :ASSUMED, from uses of the object.
-  (where-from :assumed :type (member :declared :assumed :defined-here :defined :defined-method))
+  (where-from :assumed :type (member :declared :declared-verify :assumed :defined-here :defined :defined-method))
   ;; list of the REF nodes for this leaf
   (refs () :type list)
   ;; For tracking whether to warn about unused variables:
@@ -829,9 +851,9 @@
   ;; SET if there was a set but no REF.
   ;; T if there was a REF.
   ;; This may be non-nil when REFS and SETS are null, since code can be deleted.
-  (ever-used nil :type (member nil set t))
-  ;; is it declared dynamic-extent, or truly-dynamic-extent?
-  (dynamic-extent nil :type (member nil truly-dynamic-extent dynamic-extent))
+  (ever-used nil :type (member nil set t initial-unused))
+  ;; True if declared dynamic-extent.
+  (dynamic-extent nil :type boolean)
   ;; some kind of info used by the back end
   (info nil))
 (!set-load-form-method leaf (:xc :target) :ignore-it)
@@ -871,7 +893,11 @@
 
 ;;; The GLOBAL-VAR structure represents a value hung off of the symbol
 ;;; NAME.
-(defstruct (global-var (:include basic-var) (:copier nil))
+(defstruct (global-var (:include basic-var)
+                       (:copier nil)
+                       (:constructor make-global-var (kind %source-name
+                                                      &optional where-from
+                                                                type defined-type)))
   ;; kind of variable described
   (kind (missing-arg)
         :type (member :special :global-function :global :unknown)))
@@ -908,9 +934,14 @@
 ;;; an inline proclamation) we copy the structure so that former
 ;;; INLINEP values are preserved.
 (defstruct (defined-fun (:include global-var
-                                  (where-from :defined)
-                                  (kind :global-function))
-                        (:copier nil))
+                         (where-from :defined)
+                         (kind :global-function))
+               (:constructor make-defined-fun
+                   (%source-name type &optional where-from
+                    &key kind
+                         inline-expansion inlinep
+                         same-block-p functional))
+               (:copier nil))
   ;; The values of INLINEP and INLINE-EXPANSION initialized from the
   ;; global environment.
   (inlinep nil :type inlinep)
@@ -930,14 +961,55 @@
   (functional :test functional))
 
 ;;;; function stuff
+(!def-boolean-attribute functional-kind
+  nil optional deleted external toplevel
+  escape cleanup let mv-let assignment
+  zombie toplevel-xep)
+
+(defmacro functional-kind-eq (functional &rest kinds)
+  `(,(if (cdr kinds)
+         'logtest
+         'eql)
+    (functional-kind ,functional)
+    (functional-kind-attributes ,@kinds)))
+
+(defmacro functional-kind-ecase (functional &body cases)
+  (let ((kind (gensym)))
+    `(let ((,kind (functional-kind ,functional)))
+       (cond
+         ,@(loop for (case* . forms) in cases
+                 for case = (ensure-list case*)
+                 collect
+                 `((,(if (singleton-p case)
+                         'eql
+                         'logtest) ,kind (functional-kind-attributes ,@case))
+                   ,@forms))
+         (t (error "Unhandled functional-kind ~a"
+                   (decode-functional-kind-attributes ,kind)))))))
+
+(defmacro functional-kind-case (functional &body cases)
+  (let ((kind (gensym)))
+    `(let ((,kind (functional-kind ,functional)))
+       (cond
+         ,@(loop for (case* . forms) in cases
+                 for case = (ensure-list case*)
+                 collect
+                 (if (eq case* t)
+                     `(t ,@forms)
+                     `((,(if (singleton-p case)
+                             'eql
+                             'logtest) ,kind (functional-kind-attributes ,@case))
+                       ,@forms)))))))
 
 ;;; We default the WHERE-FROM and TYPE slots to :DEFINED and FUNCTION.
 ;;; We don't normally manipulate function types for defined functions,
 ;;; but if someone wants to know, an approximation is there.
 (defstruct (functional (:include leaf
-                                 (%source-name '.anonymous.)
-                                 (where-from :defined)
-                                 (type (specifier-type 'function)))
+                        (%source-name '.anonymous.)
+                        (where-from :defined)
+                        (type (specifier-type 'function)))
+                       (:constructor make-functional (&key kind info
+                                                           %source-name %debug-name lexenv))
                        (:copier nil))
   ;; (For public access to this slot, use LEAF-DEBUG-NAME.)
   ;;
@@ -972,8 +1044,8 @@
   ;;   %SOURCE-NAME=FOO (or maybe .ANONYMOUS.?)
   ;;   %DEBUG-NAME=(MACRO-FUNCTION FOO)
   (%debug-name nil
-               :type (or null (not (satisfies legal-fun-name-p)))
-               :read-only t)
+   :type (or null (not (satisfies legal-fun-name-p)))
+   :read-only t)
   ;; some information about how this function is used. These values
   ;; are meaningful:
   ;;
@@ -1034,9 +1106,7 @@
   ;;
   ;;    :ZOMBIE
   ;;    Effectless [MV-]LET; has no BIND node.
-  (kind nil :type (member nil :optional :deleted :external :toplevel
-                          :escape :cleanup :let :mv-let :assignment
-                          :zombie :toplevel-xep))
+  (kind #.(functional-kind-attributes nil) :type attributes)
   ;; Is this a function that some external entity (e.g. the fasl dumper)
   ;; refers to, so that even when it appears to have no references, it
   ;; shouldn't be deleted? In the old days (before
@@ -1085,11 +1155,12 @@
   (inline-expanded nil :type boolean)
   ;; Is it coming from a top-level NAMED-LAMBDA?
   (top-level-defun-p nil)
-  (ignore nil))
+  (ignore nil)
+  (reanalyze nil))
 
 (defun pretty-print-functional (functional stream)
   (let ((name (functional-debug-name functional)))
-    (princ `(function
+    (prin1 `(function
              ,(if (typep name '(cons (member xep tl-xep)))
                   (cadr name)
                   name))
@@ -1115,18 +1186,16 @@
 
 ;;; Is FUNCTIONAL LET-converted? (where we're indifferent to whether
 ;;; it returns one value or multiple values)
-(defun functional-letlike-p (functional)
-  (member (functional-kind functional)
-          '(:let :mv-let)))
+(defmacro functional-letlike-p (functional)
+  `(functional-kind-eq ,functional let mv-let))
 
 ;;; Is FUNCTIONAL sorta LET-converted? (where even an :ASSIGNMENT counts)
 ;;;
 ;;; FIXME: I (WHN) don't understand this one well enough to give a good
 ;;; definition or even a good function name, it's just a literal copy
 ;;; of a CMU CL idiom. Does anyone have a better name or explanation?
-(defun functional-somewhat-letlike-p (functional)
-  (or (functional-letlike-p functional)
-      (eql (functional-kind functional) :assignment)))
+(defmacro functional-somewhat-letlike-p (functional)
+  `(functional-kind-eq ,functional let mv-let assignment))
 
 ;;; FUNCTIONAL name operations
 (defun functional-debug-name (functional)
@@ -1149,6 +1218,9 @@
 (defstruct (clambda (:include functional)
                     (:conc-name lambda-)
                     (:predicate lambda-p)
+                    (:constructor make-clambda
+                        (&key vars kind bind home %debug-name %source-name
+                              lexenv allow-instrumenting))
                     (:copier nil))
   ;; list of LAMBDA-VAR descriptors for arguments
   (vars nil :type list)
@@ -1204,7 +1276,7 @@
              :pretty-ir-printer (pretty-print-functional structure stream))
   %source-name
   %debug-name
-  kind
+  (kind :princ (decode-functional-kind-attributes kind))
   (type :test (not (eq type *universal-type*)))
   (where-from :test (not (eq where-from :assumed)))
   (vars :prin1 (mapcar #'leaf-source-name vars)))
@@ -1222,7 +1294,7 @@
 ;;; :TOPLEVEL mess with a flag COMPONENT-HAS-EXTERNAL-REFERENCES-P
 ;;; along the lines of FUNCTIONAL-HAS-EXTERNAL-REFERENCES-P.
 (defun lambda-toplevelish-p (clambda)
-  (or (eql (lambda-kind clambda) :toplevel)
+  (or (functional-kind-eq clambda toplevel)
       (lambda-has-external-references-p clambda)))
 (defun component-toplevelish-p (component)
   (member (component-kind component)
@@ -1251,7 +1323,11 @@
 ;;; arguments into a direct call to the appropriate entry-point
 ;;; function, so functions that are compiled together can avoid doing
 ;;; the dispatch.
-(defstruct (optional-dispatch (:include functional) (:copier nil))
+(defstruct (optional-dispatch
+            (:include functional) (:copier nil)
+            (:constructor make-optional-dispatch
+                (arglist allowp keyp %source-name %debug-name source-path))
+            #-sb-xc-host :no-constructor-defun)
   ;; the original parsed argument list, for anyone who cares
   (arglist nil :type list)
   ;; true if &ALLOW-OTHER-KEYS was supplied
@@ -1301,7 +1377,8 @@
 ;;; LAMBDA-VARs during IR1 conversion. If we use one of these things,
 ;;; then the var will have to be massaged a bit before it is simple
 ;;; and lexical.
-(defstruct (arg-info (:copier nil))
+(defstruct (arg-info (:copier nil)
+                     (:constructor make-arg-info (kind)))
   ;; true if this arg is to be specially bound
   (specialp nil :type boolean)
   ;; the kind of argument being described. Required args only have arg
@@ -1362,11 +1439,16 @@
   ;; Do not propagate constraints for this var
   no-constraints
   ;; Does it hold a constant that should't be destructively modified
-  constant)
+  constant
+  unused-initial-value
+  ;; Instruct constraint propagation to do some work.
+  compute-same-refs)
 
-(defstruct (lambda-var (:include basic-var) (:copier nil))
-  (flags (lambda-var-attributes)
-         :type attributes)
+(defstruct (lambda-var
+            (:include basic-var) (:copier nil)
+            (:constructor make-lambda-var
+              (%source-name &key type where-from specvar source-form arg-info)))
+  (flags #.(lambda-var-attributes) :type attributes)
   ;; the CLAMBDA that this var belongs to. This may be null when we are
   ;; building a lambda during IR1 conversion.
   (home nil :type (or null clambda))
@@ -1389,11 +1471,12 @@
   (ctype-constraints nil :type (or null hash-table))
   (eq-constraints    nil :type (or null hash-table))
   ;; sorted sets of constraints we like to iterate over
-  (eql-var-constraints     nil :type (or null (array t 1)))
-  (inheritable-constraints nil :type (or null (array t 1)))
-  (equality-constraints    nil :type (or null (array t 1)))
-
+  (eql-var-constraints     nil :type (or null (vector t)))
+  (inheritable-constraints nil :type (or null (vector t)))
+  (equality-constraints    nil :type (or null (vector t)))
+  (equality-constraints-hash nil :type (or null hash-table))
   source-form)
+
 (defprinter (lambda-var :identity t)
   %source-name
   (type :test (not (eq type *universal-type*)))
@@ -1415,6 +1498,11 @@
   `(lambda-var-attributep (lambda-var-flags ,var) no-constraints))
 (defmacro lambda-var-constant (var)
   `(lambda-var-attributep (lambda-var-flags ,var) constant))
+(defmacro lambda-var-unused-initial-value (var)
+  `(lambda-var-attributep (lambda-var-flags ,var) unused-initial-value))
+
+(defmacro lambda-var-compute-same-refs (var)
+  `(lambda-var-attributep (lambda-var-flags ,var) compute-same-refs))
 
 
 ;;;; basic node types
@@ -1433,16 +1521,25 @@
   ;; The leaf referenced.
   (leaf nil :type leaf)
   ;; KLUDGE: This is supposed to help with keyword debug messages somehow.
-  (%source-name (missing-arg) :type symbol :read-only t))
+  (%source-name (missing-arg) :type symbol :read-only t)
+  ;; An cons added by constraint-propagate to all REFs that have the
+  ;; same value when referencing a lambda-var with sets.
+  (same-refs nil :type (or null cons)))
 (defprinter (ref :identity t)
   (%source-name :test (neq %source-name '.anonymous.))
   leaf)
 
+(defstruct (multiple-successors-node
+            (:constructor nil)
+            (:include node)
+            (:copier nil)
+            (:conc-name node-)))
+
 ;;; Naturally, the IF node always appears at the end of a block.
-(defstruct (cif (:include node)
+(defstruct (cif (:include multiple-successors-node)
                 (:conc-name if-)
                 (:predicate if-p)
-                (:constructor make-if)
+                (:constructor make-if (test consequent alternative))
                 (:copier nil))
   ;; LVAR for the predicate
   (test (missing-arg) :type lvar)
@@ -1457,13 +1554,24 @@
   consequent
   alternative)
 
+(defstruct (jump-table (:include multiple-successors-node)
+                       (:constructor make-jump-table (index))
+                       (:copier nil)
+                       #-sb-xc-host :no-constructor-defun)
+  (index (missing-arg) :type lvar)
+  (targets nil :type list))
+
+(defprinter (jump-table :identity t)
+  index
+  targets)
+
 (defstruct (cset (:include valued-node
-                           (derived-type (make-single-value-type
-                                          *universal-type*)))
+                           (derived-type (make-single-value-type *universal-type*)))
                  (:conc-name set-)
                  (:predicate set-p)
-                 (:constructor make-set)
-                 (:copier nil))
+                 (:constructor make-set (var value))
+                 (:copier nil)
+                 #-sb-xc-host :no-constructor-defun)
   ;; descriptor for the variable set
   (var (missing-arg) :type basic-var)
   ;; LVAR for the value form
@@ -1513,7 +1621,11 @@
   (info nil)
   (step-info nil)
   ;; A plist of inline expansions
-  (inline-expansions *inline-expansions* :type list :read-only t))
+  (inline-expansions *inline-expansions* :type list :read-only t)
+  ;; Current COMPONENT-REOPTIMIZE-COUNTER set when calling delay-ir1-transform.
+  (delay-to -1 :type fixnum)
+  (constraints-in)
+  #+() (constraints-out))
 
 ;;; The COMBINATION node represents all normal function calls,
 ;;; including FUNCALL. This is distinct from BASIC-COMBINATION so that
@@ -1521,7 +1633,8 @@
 (defstruct (combination (:include basic-combination)
                         (:constructor make-combination (fun))
                         (:copier nil))
-  (pass-nargs t :type boolean))
+  (pass-nargs t :type boolean)
+  (or-chain-computed nil :type boolean))
 (defprinter (combination :identity t)
   (fun :prin1 (lvar-uses fun))
   (args :prin1 (mapcar (lambda (x)
@@ -1535,7 +1648,8 @@
 ;;; receiving forms.
 (defstruct (mv-combination (:include basic-combination)
                            (:constructor make-mv-combination (fun))
-                           (:copier nil)))
+                           (:copier nil)
+                           #-sb-xc-host :no-constructor-defun))
 (defprinter (mv-combination)
   (fun :prin1 (lvar-uses fun))
   (args :prin1 (mapcar #'lvar-uses args)))
@@ -1543,7 +1657,8 @@
 ;;; The BIND node marks the beginning of a lambda body and represents
 ;;; the creation and initialization of the variables.
 (defstruct (bind (:include node)
-                 (:copier nil))
+                 (:copier nil)
+                 (:constructor make-bind ()))
   ;; the lambda we are binding variables for. Null when we are
   ;; creating the LAMBDA during IR1 translation.
   (lambda nil :type (or clambda null)))
@@ -1557,8 +1672,9 @@
 (defstruct (creturn (:include node)
                     (:conc-name return-)
                     (:predicate return-p)
-                    (:constructor make-return)
-                    (:copier nil))
+                    (:constructor make-return (result lambda))
+                    (:copier nil)
+                    #-sb-xc-host :no-constructor-defun)
   ;; the lambda we are returning from. Null temporarily during
   ;; ir1tran.
   (lambda nil :type (or clambda null))
@@ -1578,7 +1694,9 @@
 ;;; type ASSERTED-TYPE.
 (defstruct (cast (:include valued-node)
                  (:copier nil)
-                 (:constructor %make-cast))
+                 (:constructor %make-cast
+                     (asserted-type type-to-check value derived-type context))
+                 #-sb-xc-host :no-constructor-defun)
   (asserted-type (missing-arg) :type ctype)
   (type-to-check (missing-arg) :type ctype)
   ;; an indication of what we have proven about how this type
@@ -1613,26 +1731,14 @@
                    (%type-check nil)
                    (asserted-type *wild-type*)
                    (type-to-check *wild-type*))
+                  (:constructor make-delay (value))
                   (:copier nil)))
 
-;;; A cast that always follows %check-bound and they are deleted together.
-;;; Created via BOUND-CAST ir1-translator by chaining it together with %check-bound.
-;;; IR1-OPTIMIZE-CAST handles propagation from BOUND to CAST-ASSERTED-TYPE
-;;; DELETE-CAST deletes BOUND-CAST-CHECK
-;;; GENERATE-TYPE-CHECKS ignores it, it never translates to a type check,
-;;; %CHECK-BOUND does all the checking.
-(defstruct (bound-cast (:include cast (%type-check nil))
-                       (:copier nil))
-  ;; %check-bound combination before the cast
-  (check (missing-arg) :type (or null combination))
-  ;; Tells whether the type information is in a state where it can be
-  ;; optimized away, i.e. when BOUND is a constant.
-  (derived nil :type boolean)
-  (array (missing-arg) :type lvar)
-  (bound (missing-arg) :type lvar))
-
 ;;; Inserted by ARRAY-CALL-TYPE-DERIVER so that it can be later deleted
-(defstruct (array-index-cast (:include cast) (:copier nil)))
+(defstruct (array-index-cast
+            (:include cast) (:copier nil)
+            (:constructor make-array-index-cast
+                (asserted-type type-to-check value derived-type))))
 
 ;;;; non-local exit support
 ;;;;
@@ -1643,7 +1749,8 @@
 ;;; lexical exit. It is the mess-up node for the corresponding :ENTRY
 ;;; cleanup.
 (defstruct (entry (:include node)
-                  (:copier nil))
+                  (:copier nil)
+                  (:constructor make-entry ()))
   ;; All of the EXIT nodes for potential non-local exits to this point.
   (exits nil :type list)
   ;; The cleanup for this entry. NULL only temporarily.
@@ -1658,7 +1765,8 @@
 ;;; lvar is the exit node's LVAR; environment analysis also makes it
 ;;; the lvar of %NLX-ENTRY call.
 (defstruct (exit (:include valued-node)
-                 (:copier nil))
+                 (:copier nil)
+                 (:constructor make-exit (&optional entry value)))
   ;; the ENTRY node that this is an exit for. If null, this is a
   ;; degenerate exit. A degenerate exit is used to "fill" an empty
   ;; block (which isn't allowed in IR1.) In a degenerate exit, Value
@@ -1675,7 +1783,8 @@
 ;;; The ENCLOSE node marks the place at which closure allocation code
 ;;; would be emitted, if necessary.
 (defstruct (enclose (:include node)
-                    (:copier nil))
+                    (:copier nil)
+                    (:constructor make-enclose (funs)))
   ;; The list of functionals that this ENCLOSE node allocates.
   (funs nil :type list)
   ;; The dynamic extent for this enclose if any of its functionals are
@@ -1690,21 +1799,19 @@
 (defstruct (cdynamic-extent (:include node)
                             (:conc-name dynamic-extent-)
                             (:predicate dynamic-extent-p)
-                            (:constructor make-dynamic-extent)
+                            (:constructor make-dynamic-extent (&optional values))
                             (:copier nil))
-  ;; the kind of dynamic extent this is.
-  (kind (missing-arg) :type (member dynamic-extent truly-dynamic-extent))
   ;; the values explicitly declared with this dynamic extent.
   (values nil :type list)
   ;; the cleanup for this extent. NULL indicates that this dynamic
   ;; extent is over the environment and hence needs no cleanup code.
   (cleanup nil :type (or cleanup null))
   ;; some kind of info used by the back end
-  (info nil))
+  (info nil)
+  (preserve-info nil))
 
 (defprinter (cdynamic-extent :conc-name dynamic-extent-
                              :identity t)
-  kind
   values
   (info :test info))
 
