@@ -19,6 +19,7 @@
 
 (defstruct (arg-state (:copier nil))
   (stack-frame-size 0))
+(declaim (freeze-type arg-state))
 
 (define-alien-type-method (integer :arg-tn) (type state)
   (let ((stack-frame-size (arg-state-stack-frame-size state)))
@@ -58,6 +59,7 @@
 
 (defstruct (result-state (:copier nil))
   (num-results 0))
+(declaim (freeze-type result-state))
 
 (defun result-reg-offset (slot)
   (ecase slot
@@ -128,8 +130,8 @@
 
 
 (deftransform %alien-funcall ((function type &rest args) * * :node node)
-  (aver (sb-c::constant-lvar-p type))
-  (let* ((type (sb-c::lvar-value type))
+  (aver (sb-c:constant-lvar-p type))
+  (let* ((type (sb-c:lvar-value type))
          (env (sb-c::node-lexenv node))
          (arg-types (alien-fun-type-arg-types type))
          (result-type (alien-fun-type-result-type type)))
@@ -205,11 +207,10 @@
   (:ignore eax)
   (:generator 1
    (inst movsx res
-         (make-random-tn :kind :normal
-                         :sc (sc-or-lose (ecase size
-                                           (8 'byte-reg)
-                                           (16 'word-reg)))
-                         :offset (tn-offset val)))))
+         (make-random-tn (sc-or-lose (ecase size
+                                       (8 'byte-reg)
+                                       (16 'word-reg)))
+           (tn-offset val)))))
 
 #-sb-xc-host
 (defun sign-extend (x size)
@@ -229,7 +230,6 @@
   (:generator 2
    (inst mov res (make-fixup foreign-symbol :foreign))))
 
-#+linkage-table
 (define-vop (foreign-symbol-dataref-sap)
   (:translate foreign-symbol-dataref-sap)
   (:policy :fast-safe)
@@ -286,7 +286,8 @@
            (inst lea edi (make-ea :dword :base ebp-tn
                                   :disp (frame-byte-offset (tn-offset pc-save))))
            (move eax function)
-           (inst call (make-fixup "call_into_c" :foreign))
+           (pseudo-atomic (:elide-if (not (call-out-pseudo-atomic-p vop)))
+             (inst call (make-fixup "call_into_c" :foreign)))
            (when (and results
                       (location= (tn-ref-tn results) fr0-tn))
              (force-x87-to-mem (tn-ref-tn results) fp-temp)))
@@ -300,7 +301,8 @@
            ;; this, and it should not hurt others either.
            (inst cld)
 
-           (inst call function)
+           (pseudo-atomic (:elide-if (not (call-out-pseudo-atomic-p vop)))
+             (inst call function))
            ;; To give the debugger a clue. FIXME: not really internal-error?
            (note-this-location vop :internal-error)
 
@@ -378,17 +380,6 @@
               delta)))
     (load-symbol-value result *alien-stack-pointer*)))
 
-;;; not strictly part of the c-call convention, but needed for the
-;;; WITH-PINNED-OBJECTS macro used for "locking down" lisp objects so
-;;; that GC won't move them while foreign functions go to work.
-(define-vop (touch-object)
-  (:translate touch-object)
-  (:args (object))
-  (:ignore object)
-  (:policy :fast-safe)
-  (:arg-types t)
-  (:generator 0))
-
 #-sb-xc-host
 (defun alien-callback-accessor-form (type sp offset)
   `(deref (sap-alien (sap+ ,sp ,offset) (* ,type))))
@@ -452,7 +443,7 @@ pointer to the arguments."
     (finalize-segment segment)
     ;; Now that the segment is done, convert it to a static
     ;; vector we can point foreign code to.
-    (let ((buffer (sb-assem::segment-buffer segment)))
+    (let ((buffer (sb-assem:segment-buffer segment)))
       (make-static-vector (length buffer)
                           :element-type '(unsigned-byte 8)
                           :initial-contents buffer))))
