@@ -1778,13 +1778,14 @@
    (integer 4 4)))
 
 (with-test (:name :inherit-length-var)
-  (assert (not (ctu:ir1-named-calls
-                `(lambda (x y n)
-                   (declare (optimize (space 0))
-                            (fixnum n)
-                            (simple-base-string y))
-                   (when (< n (length y))
-                     (find x y :start n))))))
+  (assert (not (find 'sb-int:sequence-bounding-indices-bad-error
+                     (ctu:ir1-named-calls
+                      `(lambda (x y n)
+                         (declare (optimize (space 0))
+                                  (fixnum n)
+                                  (simple-base-string y))
+                         (when (< n (length y))
+                           (find x y :start n)))))))
   (assert-type
    (lambda (x y)
      (declare (simple-vector x))
@@ -1863,7 +1864,8 @@
          m))
    (or double-float null)))
 
-(with-test (:name :ignore-delays)
+(with-test (:name :ignore-delays
+            :fails-on :arm)
   (assert-type
    (lambda (x)
      (declare (optimize debug))
@@ -1877,3 +1879,88 @@
      (and (eql a b)
           (not (eq a b))))
    boolean))
+
+(with-test (:name :equal-no-notes)
+  (checked-compile
+   `(lambda (a b)
+      (if (eq a b)
+          (equal a b)
+          nil))
+   :allow-notes nil))
+
+(with-test (:name :contagion-alt)
+  (assert-type
+   (lambda (x y)
+     (declare (real x y))
+     (if (typep (+ x y) 'double-float)
+         (error "")
+         x))
+   (or rational single-float))
+  (assert-type
+   (lambda (x)
+     (declare (real x))
+     (if (typep (+ x 1.0) 'single-float)
+         (error "")
+         x))
+   double-float)
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1.0) 'single-float)
+         (error "")
+         x))
+   (and number (not rational) (not single-float)))
+  (assert-type
+   (lambda (x)
+     (declare (real x))
+     (if (typep (+ x 1) 'single-float)
+         (error "")
+         x))
+   (or double-float rational))
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1) 'single-float)
+         (error "")
+         x))
+   (and number (not single-float)))
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1) 'double-float)
+         (error "")
+         x))
+   (and number (not double-float)))
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1d0) 'double-float)
+         (error "")
+         x))
+   complex)
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1.0) 'double-float)
+         (error "")
+         x))
+   (and number (not double-float)))
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1.0) 'real)
+         (error "")
+         x))
+   (and number (not real)))
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1.0) 'float)
+         (error "")
+         x))
+   (and number (not real)))
+  (assert-type
+   (lambda (x)
+     (if (typep (+ x 1) 'float)
+         (error "")
+         x))
+   (and number (not float)))
+  (assert-type
+   (lambda (x)
+     (if (typep (* x 3) 'integer)
+         (error "")
+         x))
+   (and number (not integer))))
