@@ -44,14 +44,14 @@
 ;;;; fork/exec, so that no lisp is on the stack. This is our known-good
 ;;;; number.
 
-(defvar *exename* (format nil "stackalign-test~a" (or #+win32 ".exe" "")))
-(defvar *soname*  (format nil "stackalign-test~a" (or #+win32 ".dll" ".so")))
+(defvar *exename* (scratch-file-name (or #+win32 "exe" nil)))
+(defvar *soname*  (scratch-file-name (or #+win32 "dll" "so")))
 
 (progn
   (cc #+unix "-sbcl-pic" "-o" *exename* "stack-alignment-offset.c")
 
   (defparameter *good-offset*
-    (parse-integer (run (format nil "./~a"  *exename*)
+    (parse-integer (run *exename*
                         (princ-to-string *required-alignment*))))
   (format t "~s is ~d~%" '*good-offset* *good-offset*)
   ;; Build the tool again, this time as a shared object, and load it
@@ -59,7 +59,7 @@
   #+unix  (cc "-sbcl-shared" "-sbcl-pic" "-o" *soname* "stack-alignment-offset.c")
   #+win32 (cc "-shared" "-o" *soname* "stack-alignment-offset.c")
 
-  (load-shared-object (truename *soname*))
+  (load-shared-object *soname*)
 
   (define-alien-routine stack-alignment-offset int (alignment int))
   #+alien-callbacks
