@@ -13,8 +13,29 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(load "assertoid.lisp")
-(use-package "ASSERTOID")
+;;; Test that symbols are properly normalized in SB-UNICODE builds
+#+sb-unicode
+(with-test (:name (:normalizing-reader)
+                  :skipped-on (not :sb-unicode))
+  (labels ((str (&rest chars)
+             (coerce chars 'string))
+           (symbol (&rest chars)
+             (read-from-string (apply #'str chars))))
+    (assert (eq :a :a))
+    (assert (eq :a :A))
+    (assert (eq (symbol #\UF984) (symbol #\U6FFE)))
+    (make-package "BAFFLE")
+    (intern "C" "BAFFLE")
+    (assert (eq (symbol #\b #\a #\f #\f #\l #\e #\: #\: #\c)
+                (symbol #\b #\a #\UFB04 #\e #\: #\: #\c)))
+    (assert (not (eq (symbol #\| #\f #\f #\l #\|) (symbol #\| #\UFB04 #\|))))
+    (assert (not (eq (symbol #\\ #\U32C0) (symbol #\1 #\U6708))))
+    (assert (eq (symbol #\U32C0) (symbol #\1 #\U6708)))
+    (let ((*readtable* (copy-readtable)))
+      (setf (sb-ext:readtable-normalization *readtable*) nil)
+      (assert (not (eq (symbol #\b #\a #\f #\f #\l #\e)
+                       (symbol #\b #\a #\UFB04 #\e))))
+      (assert (not (eq (symbol #\U32C0) (symbol #\1 #\U6708)))))))
 
 ;;; Bug 30, involving mistakes in binding the read table, made this
 ;;; code fail.
@@ -38,8 +59,8 @@
 
 ;;; Bug 51b. (try to throw READER-ERRORs when the reader encounters
 ;;; dubious input)
-(assert (raises-error? (read-from-string "1e1000") reader-error))
-(assert (raises-error? (read-from-string "1/0") reader-error))
+(assert-error (read-from-string "1e1000") reader-error)
+(assert-error (read-from-string "1/0") reader-error)
 
 ;;; Bug reported by Antonio Martinez on comp.lang.lisp 2003-02-03 in
 ;;; message <b32da960.0302030640.7d6fc610@posting.google.com>: reading
@@ -81,7 +102,8 @@
 (defstruct readable-struct a)
 (macrolet
     ((frob (string)
-       `(assert (eq (readable-struct-a (read-from-string ,string)) t))))
+       `(handler-bind ((warning #'muffle-warning))
+          (assert (eq (readable-struct-a (read-from-string ,string)) t)))))
   (frob "#S(READABLE-STRUCT :A T)")
   (frob "#S(READABLE-STRUCT A T)")
   (frob "#S(READABLE-STRUCT \"A\" T)")
@@ -89,7 +111,7 @@
   (frob "#S(READABLE-STRUCT #\\A T :A NIL)"))
 (macrolet
     ((frob (string)
-       `(assert (raises-error? (read-from-string ,string) reader-error))))
+       `(assert-error (read-from-string ,string) reader-error)))
   (frob "#S(READABLE-STRUCT . :A)")
   (frob "#S(READABLE-STRUCT :A . T)")
   (frob "#S(READABLE-STRUCT :A T . :A)")
@@ -120,7 +142,7 @@
 (assert (eq 'a (read (make-instance 'my-in-stream :last-char nil))))
 
 ;;; NIL as the last argument to SET-SYNTAX-FROM-CHAR in compiled code,
-;;; reported by Levente Mszros
+;;; reported by Levente Mészáros
 (let ((fun (compile nil '(lambda ()
                           (set-syntax-from-char #\{ #\( *readtable* nil)))))
   (funcall fun)
@@ -168,12 +190,7 @@
     (assert (eq (find-package :cl) (test "cl:no-such-sym")))))
 
 ;; lp# 1012335 - also tested by 'READ-BOX above
-(handler-bind ((condition #'continue))
-    (defun nil (stream char) (declare (ignore stream char)) 'foo!))
 (with-test (:name :set-macro-char-lazy-coerce-to-fun)
-  (set-macro-character #\$ #'nil) ; #'NIL is a function
-  (assert (eq (read-from-string "$") 'foo!))
-
   (make-dispatch-macro-character #\$)
   (assert (set-dispatch-macro-character #\$ #\( 'read-metavar))
   (assert (eq (get-dispatch-macro-character #\$ #\() 'read-metavar))
@@ -201,11 +218,12 @@
                 :ok))))
 
 (defun cl-user::esoteric-load-thing ()
-  ;; This LOAD-AS-SOURCE will fail if PRINT1 reads as the keyword :PRIN1
+  ;; This LOAD-AS-SOURCE will fail if SET reads as the keyword :SET
   (let ((s (make-string-input-stream
-            "(cl:in-package :cl-user) (prin1 'okey-dokey)")))
+            "(cl:in-package :cl-user) (set 'okey-dokey 3)")))
     (let ((*package* *package*))
-      (sb-impl::load-as-source s :print t :verbose t))))
+      (sb-impl::load-as-source s :print nil :verbose nil))
+    (assert (eql (symbol-value 'cl-user::okey-dokey) 3))))
 
 (with-test (:name :reader-package-in-conditional)
   ;; Sharp-plus binds *package* but not *reader-package* so that if,
@@ -219,6 +237,10 @@
 
 #+sb-unicode
 (with-test (:name :unicode-dispatch-macros)
+  ;; Smoke test: (set-syntax-from-char unicode-char ordinary-constituent-char)
+  ;; should not fail
+  (set-syntax-from-char (code-char 300) #\a)
+  ;;
   (let ((*readtable* (copy-readtable)))
     (make-dispatch-macro-character (code-char #x266F)) ; musical sharp
     (set-dispatch-macro-character
@@ -229,9 +251,12 @@
     (let ((x (read-from-string
               (map 'string #'code-char '(#x266F #x221E)))))
       (assert (eq x :infinity))
+      ;; I don't know what this was testing, and it's "noisy". Can we fix that?
+      ;; I think we used to treat NIL as *removing* the macro function, which is not
+      ;; a specified action. But neither could NIL ever be a function designator.
       (set-dispatch-macro-character (code-char #x266F) (code-char #x221E) nil)
       (assert (zerop (hash-table-count
-                      (car (sb-impl::%dispatch-macro-char-table
+                      (cdr (sb-impl::%dispatch-macro-char-table
                             (get-macro-character (code-char #x266F)))))))))
 
   (let ((*readtable* (copy-readtable)))
@@ -312,4 +337,17 @@
       (assert (eq t (set-dispatch-macro-character #\# #\~ fun nil)))
       (assert (eq fun (get-dispatch-macro-character #\# #\~ nil))))))
 
+(defclass junk () (a))
+(defstruct foo a b)
+(with-test (:name :sharp=-visit-unbound-slot-no-crash)
+  (unwind-protect
+       (progn
+         (sb-int:encapsulate 'sb-int:add-to-xset 'wrap
+          (compile nil
+                   '(lambda (realfun elt xset)
+                     (cond ((sb-int:unbound-marker-p elt) (error "oh no"))
+                           (t (funcall realfun elt xset))))))
+         (read-from-string "#1=#S(FOO :A #.(MAKE-INSTANCE 'junk))"))
+    (sb-int:unencapsulate 'sb-int:add-to-xset 'wrap)))
+
 ;;; success
